﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Data.OleDb;
using System.Data.Sql;
using System.Drawing.Imaging;
using System.Security.Cryptography;
using System.Data.Common;

namespace CatalogConverter
{
    public partial class FormCatalogConverter : Form
    {
        public FormCatalogConverter()
        {
            InitializeComponent();
        }

        private void buttonSelectInput_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            //openFileDialog1.InitialDirectory = @"C:\";
            openFileDialog1.InitialDirectory = this.textBoxInput.Text;
            openFileDialog1.Title = "Open Catalog";
            openFileDialog1.DefaultExt = "txt";
            openFileDialog1.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
            openFileDialog1.FilterIndex = 2;
            openFileDialog1.CheckFileExists = true;
            openFileDialog1.CheckPathExists = true;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.textBoxInput.Text = openFileDialog1.FileName;
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "Selected File: " + openFileDialog1.FileName;
            }
        }

        private void buttonGenerate_Click(object sender, EventArgs e)
        {
            this.labelStatus.Text = "Status: RUNNING...";
            this.labelStatus.ForeColor = Color.Red;
            this.labelStatus.Refresh();

            if (this.radioButtonKrakow.Checked)
            {
                Krakow();
            }
            if (this.radioButtonPaschke.Checked)
            {
                Paschke();
            }
            if (this.radioButtonPredpovedi.Checked)
            {
                Predpovedi();
            }
            else
            {
                this.richTextBoxOutput.Text = "No option Selected";
            }
            
            this.labelStatus.Text = "Status: DONE !";
            this.labelStatus.ForeColor = Color.Green;
            this.labelStatus.Refresh();
        }

        private void Krakow()
        {
            try
            {
                string filename = Path.GetFileNameWithoutExtension(this.textBoxInput.Text);
                string directoryname = Path.GetDirectoryName(this.textBoxInput.Text);
                string outputfile1 = directoryname + @"\" + filename + "1.dat";
                string outputfile2 = directoryname + @"\" + filename + ".cat";

                StreamWriter fileWriter1 = new StreamWriter(outputfile1, true);
                //StreamWriter fileWriter2 = new StreamWriter(outputfile2, true);

                StreamReader file = new StreamReader(this.textBoxInput.Text);

                string header = @" NAZEV        TYP BODY RA2000   DE2000  M0          PER          MAX   PRIM  SEK   OB ZDROJ                 VD   MD   MAPA     ATRAKTIV   POZNAMKA        GSC         ZMENA ";
                this.richTextBoxOutput.Text = header + "\n";
                fileWriter1.WriteLine(header);

                string line;
                int linenr = 0;
                string lastline = "";
                while ((line = file.ReadLine()) != null)
                {
                    linenr++;
                    if (linenr > 17 && linenr % 2 == 0)
                    {
                        string resultline = "";

                        string firstline = lastline;
                        string secondline = line;

                        string[] firstlineList = lastline.Split(new char[0], StringSplitOptions.RemoveEmptyEntries);
                        string[] secondlineList = line.Split(new char[0], StringSplitOptions.RemoveEmptyEntries);

                        //1 tot 1
                        resultline = resultline + " ";

                        //2 13  NAME  Star's designation. Colon in the name denotes that data concern to secondary minimum of the variable.  
                        string name;
                        if (firstline.Substring(47, 3) == "SEC")
                        {
                            name = firstlineList[0].ToString() + ":" + firstlineList[1].ToString();
                        }
                        else
                        {
                            name = firstlineList[0].ToString() + " " + firstlineList[1].ToString();
                        }
                        name = name.PadRight(12);
                        resultline = resultline + name + " ";

                        //15 17  TYPE The categorization of the star in B.R.N.O.observational program. For custom catalogues, you can optionally enter any text. Note: type of variability is stored in the VD.  
                        string type = "10";
                        type = type.PadRight(3);
                        resultline = resultline + type;
                        resultline = resultline + " ";

                        //19 22  RATING Given ratings are measures of usefulness of mainly visual observations.
                        string rating = "10";
                        rating = rating.PadLeft(4);
                        resultline = resultline + rating;
                        resultline = resultline + " ";

                        //24 31  RA2000 Equatorial co - ordinates in equinox 2000.0 - right ascension(hhmmss.s)
                        string ra2000;
                        ra2000 = secondlineList[0].ToString().PadLeft(2, '0') + secondlineList[1].ToString().PadLeft(2, '0') + secondlineList[2].ToString().Split('.')[0].ToString().PadLeft(2, '0') + "." + secondlineList[2].ToString().Split('.')[1].ToString().PadLeft(1, '0');
                        resultline = resultline + ra2000;
                        resultline = resultline + " ";

                        //33 39  DE2000 Equatorial co - ordinates in equinox 2000.0 - declination(+ddmmss)
                        string dec2000;
                        dec2000 = secondlineList[3].ToString().PadLeft(2, '0') + secondlineList[4].ToString().PadLeft(2, '0') + secondlineList[5].ToString().Split('.')[0].ToString().PadLeft(2, '0');
                        resultline = resultline + dec2000;
                        resultline = resultline + " ";

                        //41 51  M0 One of light elements represented by the Julian date of the basic light minimum(epoch E = 0) with the front - end figures 24... omitted.
                        string M0;
                        M0 = secondlineList[10].ToString().Substring(0, 10).PadRight(11, '0');
                        resultline = resultline + M0;
                        resultline = resultline + " ";

                        //53 64  PER Period P of light changes in days
                        string PER;
                        PER = secondlineList[9].ToString().Substring(0, 11).PadRight(12, '0');
                        resultline = resultline + PER;
                        resultline = resultline + " ";

                        //66 70  MAX Brightness of variable star in maximum(in magnitudes)
                        string MAX;
                        MAX = firstlineList[2].ToString().Split('.')[0].PadLeft(2, ' ') + "." + firstlineList[2].ToString().Split('.')[1].Substring(0, 2).PadRight(2, '0');
                        resultline = resultline + MAX;
                        resultline = resultline + " ";

                        //72 76  PRIM Brightness of variable star in primary light minimum(in magnitudes)
                        string PRIM;
                        PRIM = firstlineList[4].ToString().Split('.')[0].PadLeft(2, ' ') + "." + firstlineList[4].ToString().Split('.')[1].Substring(0, 2).PadRight(2, '0');
                        resultline = resultline + PRIM;
                        resultline = resultline + " ";

                        //78 82  SEK Brightness of variable star in secondary light minimum(in magnitudes)
                        string SEK;
                        SEK = firstlineList[4].ToString().Split('.')[0].PadLeft(2, ' ') + "." + firstlineList[4].ToString().Split('.')[1].Substring(0, 2).PadRight(2, '0');
                        resultline = resultline + SEK;
                        resultline = resultline + " ";

                        //84 85  BRANCH The photometric system in which Max, Min and Sec are reported. The main codes are P(photographic magnitudes), V(visual, photovisual, or Johnson's V), "Rc" (R magnitudes in Cousins' I system), "Hp"(Hipparcos magnitudes) and C(unfiltered CCD observations)
                        string BRANCH;
                        BRANCH = "V".PadRight(2);
                        resultline = resultline + BRANCH;
                        resultline = resultline + " ";

                        //87 107  ZDROJ Source of elements of light changes
                        string ZDROJ;
                        ZDROJ = "TIDAK".PadRight(21);
                        resultline = resultline + ZDROJ;
                        resultline = resultline + " ";

                        //109 112  VD Duration of eclipse in hours(for Algol like stars) ; the letter "B" or "W" is given for b Lyrae stars or W UMa stars, respectively.The sign "*" means variability of given value(see Notes).The sign ":" indicates possible uncertainty in given value.
                        string VD;
                        VD = "EB".PadRight(4);
                        resultline = resultline + VD;
                        resultline = resultline + " ";

                        //114 117  MD  Duration of constant phase in minimum light in hours.The sign ":" indicates potential uncertainty in given value
                        string MD;
                        MD = "0.0".PadRight(4);
                        resultline = resultline + MD;
                        resultline = resultline + " ";

                        //119 126  CHART  Availability of the chart.
                        string CHART;
                        CHART = "PROSPER".PadRight(8);
                        resultline = resultline + CHART;
                        resultline = resultline + " ";

                        //128 137  ATRAKTIV  Astrophysical attractiveness of system.

                        //139 153  POZNAMKA  Remarks.

                        //155 165  GSC  The Guide Star Catalogue identification number is given in form XXXXX.XXXXX.

                        //167 171  ZMENA  The sign "*" means change in light elements relative to previous catalogue BRKA 2004 Z and "N" means the new added star.




                        //string objname = lastline.Substring(0, 9);
                        //objname= Regex.Replace(objname, @"\s+", " ");



                        //this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + resultline + "\n";
                        this.richTextBoxOutput.Text = resultline + "\n";
                        this.richTextBoxOutput.Refresh();
                        fileWriter1.WriteLine(resultline);
                    }
                    lastline = line;

                    //if (linenr > 626) break;
                }

                this.richTextBoxOutput.Text = "DONE !" + "\n";

                fileWriter1.Flush();
                fileWriter1.Close();
                fileWriter1.Dispose();
                fileWriter1 = null;

                file.Close();
                file.Dispose();
                file = null;
            }
            catch (Exception ex)
            {
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\n" + ex.ToString();
            }
        }

        private void Paschke()
        {
            string cons_backup = "";
            string starname_backup = "";
            try
            {
                string filename = Path.GetFileNameWithoutExtension(this.textBoxInput.Text);
                string directoryname = Path.GetDirectoryName(this.textBoxInput.Text);
                string outputfile1 = directoryname + @"\" + filename + "paschke.dat";
                string outputfile2 = directoryname + @"\" + filename + "errorpaschke.txt";

                StreamWriter fileWriter1 = new StreamWriter(outputfile1, true);
                StreamWriter fileWriter2 = new StreamWriter(outputfile2, true);

                //StreamReader file = new StreamReader(this.textBoxInput.Text);

                string header = @" NAZEV        TYP BODY RA2000   DE2000  M0          PER          MAX   PRIM  SEK   OB ZDROJ                 VD   MD   MAPA     ATRAKTIV   POZNAMKA        GSC         ZMENA ";
                this.richTextBoxOutput.Text = header + "\n";
                fileWriter1.WriteLine(header);

                string resultline = "";

                OleDbConnection con; con = new OleDbConnection("Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + Application.StartupPath + "\\db\\Paschke.accdb;");
                con.Open();
                //string strqry = "SELECT ID,ObjName,RAh,RAm,RAs,Decd,Decm,Decs FROM Objects;";
                string strqry = "SELECT CONS, STARNAME, RH1, RM1, RS1, DSI1, DD1, DM1, DS1, TYPE, GSC_ID, NOTE_ID FROM IDENTIF WHERE OCGATE>#01/01/1900#;";
                OleDbCommand cmd = new OleDbCommand(strqry, con);
                cmd.CommandText = strqry;

                //Read in all elements in IDENTIF
                using (OleDbDataReader reader = cmd.ExecuteReader())
                {
                    if (reader.HasRows)
                    {
                        this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\nDatabase entries:\n";
                        
                        int nr=0;
                        while (reader.Read() && nr<10000000)
                        {
                            //int id = reader.GetInt32(0);
                            //string name = reader.GetString(1);



                            //double ra = reader.GetInt32(2) + (reader.GetInt32(3) / 60) + (reader.GetDouble(4) / 3600);
                            //double dec = reader.GetInt32(5) + (reader.GetInt32(6) / 60) + (reader.GetDouble(7) / 3600);

                            //Tuple<int, string, int, int> obj = new Tuple<int, string, int, int>(id, name, ra, dec);
                            //objlist.Add(new Tuple<int, string, double, double>(id, name, ra, dec));
                            //objlist.Add(Tuple.Create(id, name, ra, dec));

                            string cons;
                            if ((reader.GetValue(0) != DBNull.Value) && !(reader.GetValue(0) is DbException))
                                cons = Convert.ToString(reader.GetValue(0));
                            else
                                cons = "XX";
                            string starname;
                            if ((reader.GetValue(1) != DBNull.Value) && !(reader.GetValue(1) is DbException))
                                starname = Convert.ToString(reader.GetValue(1));
                            else
                                starname = "XX";
                            double rh1;
                            if ((reader.GetValue(2) != DBNull.Value) && !(reader.GetValue(2) is DbException))
                                rh1 = Convert.ToDouble(reader.GetValue(2));
                            else
                                rh1 = Convert.ToDouble("");
                            double rm1;
                            if ((reader.GetValue(3) != DBNull.Value) && !(reader.GetValue(3) is DbException))
                                rm1 = Convert.ToDouble(reader.GetValue(3));
                            else
                                rm1 = Convert.ToDouble("");
                            double rs1;
                            if ((reader.GetValue(4) != DBNull.Value) && !(reader.GetValue(4) is DbException))
                                rs1 = Convert.ToDouble(reader.GetValue(4));
                            else
                                rs1= Convert.ToDouble("");
                            string dsi1;
                            if ((reader.GetValue(5) != DBNull.Value) && !(reader.GetValue(5) is DbException))
                                dsi1 = Convert.ToString(reader.GetValue(5));
                            else
                                dsi1 = "+";
                            double dd1;
                            if ((reader.GetValue(6) != DBNull.Value) && !(reader.GetValue(6) is DbException))
                                dd1 = Convert.ToDouble(reader.GetValue(6));
                            else
                                dd1 = Convert.ToDouble("");
                            double dm1;
                            if ((reader.GetValue(7) != DBNull.Value) && !(reader.GetValue(7) is DbException))
                                dm1 = Convert.ToDouble(reader.GetValue(7));
                            else
                                dm1 = Convert.ToDouble("");
                            double ds1;
                            if ((reader.GetValue(8) != DBNull.Value) && !(reader.GetValue(8) is DbException))
                                ds1 = Convert.ToDouble(reader.GetValue(8));
                            else
                                ds1 = Convert.ToDouble("");
                            string type;
                            if ((reader.GetValue(9) != DBNull.Value) && !(reader.GetValue(9) is DbException))
                                type = Convert.ToString(reader.GetValue(9));
                            else
                                type = "XX";
                            double gsc_id;
                            if ((reader.GetValue(10) != DBNull.Value) && !(reader.GetValue(10) is DbException))
                                gsc_id = Convert.ToDouble(reader.GetValue(10));
                            else
                                gsc_id = Convert.ToDouble("");
                            string note_id;
                            if ((reader.GetValue(11) != DBNull.Value) && !(reader.GetValue(11) is DbException))
                                note_id = Convert.ToString(reader.GetValue(11));
                            else
                                note_id = "";

                            starname_backup = starname;
                            cons_backup = cons;

                            this.richTextBoxOutput.Text = richTextBoxOutput.Text + "starname:" + starname + "cons:" + cons + "rh1:" + rh1 + "rm1:" + rm1 + "rs1" + rs1;
                            //fileWriter1.WriteLine("starname:" + starname + "cons:" + cons + "rh1:" + rh1 + "rm1:" + rm1 + "rs1" + rs1);

                            double maxp = 99.9;
                            double minp = 99.9;
                            double mins = 99.9;
                            double CD=0.0;                  //hours
                            double SD=0.0;                  //hours
                            DateTime MODIFIED_BRIGHT=DateTime.MinValue;

                            string strqryb = "SELECT MAXP, MINP, MINS, CD, SD, MODIFIED FROM BRIGHT WHERE CONS='" + cons + "' AND STARNAME='" + starname + "';";
                            //fileWriter1.WriteLine("strqryb:"+strqryb);
                            OleDbCommand cmdb = new OleDbCommand(strqryb, con);
                            cmdb.CommandText = strqryb;

                            //Read in elements of BRIGHT
                            using (OleDbDataReader readerb = cmdb.ExecuteReader())
                            {
                                if (readerb.HasRows)
                                {
                                    //this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\nDatabase entries:\n";
                                    while (readerb.Read())
                                    {
                                        //Assuming there is only 1 result, otherwise take last entry for BRIGHT table
                                        if ((readerb.GetValue(0) != DBNull.Value) && !(readerb.GetValue(0) is DbException))
                                            maxp = Convert.ToDouble(readerb.GetValue(0));
                                        else
                                            maxp = 99.9;
                                        if ((readerb.GetValue(1) != DBNull.Value) && !(readerb.GetValue(1) is DbException))
                                            minp = Convert.ToDouble(readerb.GetValue(1));
                                        else
                                            minp = 99.9;
                                        if ((readerb.GetValue(2) != DBNull.Value) && !(readerb.GetValue(2) is DbException))
                                            mins = Convert.ToDouble(readerb.GetValue(2));
                                        else
                                            mins = 99.9;
                                        if ((readerb.GetValue(3) != DBNull.Value) && !(readerb.GetValue(3) is DbException))
                                            CD = Convert.ToDouble(readerb.GetValue(3));
                                        else
                                            CD = 0.0;   
                                        if ((readerb.GetValue(4) != DBNull.Value) && !(readerb.GetValue(4) is DbException))
                                            SD = Convert.ToDouble(readerb.GetValue(4));
                                        else
                                            SD = 0.0;
                                        if ((readerb.GetValue(5) != DBNull.Value) && !(readerb.GetValue(5) is DbException))
                                            MODIFIED_BRIGHT = Convert.ToDateTime(readerb.GetValue(5));
                                        //else
                                        //    MODIFIED_BRIGHT = 0.0;
                                    }
                                }
                                else
                                {
                                    //BRIGHT has no rows
                                    richTextBoxOutput.Text = "No results found in BRIGHT for "+starname_backup+" "+cons_backup;
                                    fileWriter2.WriteLine("No results found in BRIGHT for "+starname_backup+" "+cons_backup);
                                }

                                readerb.Close();
                            }

                            string strqrye = "SELECT KIND, MINIMUM9, PERIOD, ELSOUR, NOTE_EL, MODIFIED FROM ELEMENT WHERE CONS='" + cons + "' AND STARNAME='" + starname + "';";
                            //fileWriter1.WriteLine("strqrye:" + strqrye);
                            OleDbCommand cmde = new OleDbCommand(strqrye, con);
                            cmde.CommandText = strqrye;

                            //Read in elements of ELEMENT
                            using (OleDbDataReader readere = cmde.ExecuteReader())
                            {
                                if (readere.HasRows)
                                {
                                    //this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\nDatabase entries:\n";
                                    int R = 0;
                                    while (readere.Read())
                                    {
                                        //ISSUE: multiple elements for 1 object: p, s, R
                                        //narrowing down ...

                                        //1 till 1
                                        resultline = resultline + " ";

                                        //2 13  NAME  Star's designation. Colon in the name denotes that data concern to secondary minimum of the variable.  
                                        string name;
                                        /*
                                        if (firstline.Substring(47, 3) == "SEC")
                                        {
                                            name = firstlineList[0].ToString() + ":" + firstlineList[1].ToString();
                                        }
                                        else
                                        {
                                            name = firstlineList[0].ToString() + " " + firstlineList[1].ToString();
                                        }
                                        */

                                        string kind;
                                        kind = Convert.ToString(readere.GetValue(0));

                                        /*
                                        if (kind == "p")
                                        {
                                            name = starname + " " + cons;
                                        }
                                        if (kind == "s")
                                        {
                                            name = starname + ":" + cons;
                                        }
                                        if ((kind == "R") || (kind == "r"))
                                        {
                                            name = starname + " " + cons;
                                        }
                                        else
                                        {
                                            name = starname + "X" + cons;
                                        }
                                        */

                                        switch (kind)
                                        {
                                            case "p":
                                                name = starname + " " + cons;
                                                break;
                                            case "s":
                                                int breakoff = 8;
                                                if (starname.Length > breakoff)
                                                {
                                                    name = "";
                                                    bool prev_is_letter=true;
                                                    bool period_taken = false;
                                                    for (int i=0; i<starname.Length; i++)
                                                    {
                                                        char c = starname[i];
                                                        if (Char.IsLetter(c))
                                                        {
                                                            name = name + c.ToString();
                                                            prev_is_letter = true;
                                                        }
                                                        else
                                                        {
                                                            if (prev_is_letter && !period_taken)
                                                            {
                                                                //change letter to nr: add :
                                                                name = name + ":" + c.ToString();
                                                                period_taken = true;
                                                            }
                                                            if (i==(breakoff-1) && !period_taken)
                                                            {
                                                                //last character: add : anyway
                                                                name = name + ":" + c.ToString();
                                                                period_taken = true;
                                                            }
                                                            if (prev_is_letter && period_taken)
                                                            {
                                                                //: is already taken, write nr
                                                                name = name + c.ToString();
                                                            }
                                                            if (!prev_is_letter && period_taken)
                                                            {
                                                                //last character not number and : already taken
                                                                name = name + c.ToString();
                                                            }
                                                            prev_is_letter = false;
                                                        }

                                                    }
                                                }
                                                else
                                                {
                                                    name = starname + ":" + cons;
                                                }
                                                break;
                                            case "R":
                                                name = starname + " " + cons;
                                                R++;
                                                break;
                                            case "r":
                                                name = starname + " " + cons;
                                                R++;
                                                break;
                                            default:
                                                name = starname + " " + cons;
                                                break;

                                        }

                                        name = name.PadRight(12);
                                        name = name.Substring(0, Math.Min(12, name.ToString().Length));
                                        resultline = resultline + name + " ";

                                        //15 17  TYPE The categorization of the star in B.R.N.O.observational program. For custom catalogues, you can optionally enter any text. Note: type of variability is stored in the VD.  
                                        //string type = "10";
                                        switch (type.Substring(0, Math.Min(2, type.ToString().Length)))
                                        {
                                            case "EA":
                                                type = "EA";
                                                break;
                                            case "EB":
                                                type = "EB";
                                                break;
                                            case "EC":
                                                type = "EC";
                                                break;
                                            case "ED":
                                                type = "ED";
                                                break;
                                            case "EW":
                                                type = "EW";
                                                break;
                                            case "RR":
                                                type = "RR";
                                                break;
                                            default:
                                                switch (type)
                                                {
                                                    case "HADS":
                                                        type = "HDS";
                                                        break;
                                                    case "PUL":
                                                        type = "PUL";
                                                        break;
                                                    case "CEP":
                                                        type = "CEP";
                                                        break;
                                                    case "DSCT":
                                                        type = "DST";
                                                        break;
                                                    default:
                                                        type = "ntd";       //not defined...
                                                        break;
                                                }
                                                //
                                                break;
                                        }
                                        type = type.Substring(0, Math.Min(3, type.ToString().Length));
                                        type = type.PadRight(3);
                                        resultline = resultline + type;
                                        resultline = resultline + " ";

                                        //19 22  RATING Given ratings are measures of usefulness of mainly visual observations.
                                        string rating = "10";
                                        //DateTime MODIFIED_ELEMENT=DateTime.MinValue;
                                        //if ((readere.GetValue(5) != DBNull.Value) && !(readere.GetValue(5) is DbException))
                                        //    MODIFIED_ELEMENT = Convert.ToDateTime(readere.GetValue(5));
                                        //else
                                        //    MODIFIED_ELEMENT = 0.0;
                                        //ALGORITHM: select the highest change and assign 1 passed year the field
                                        
                                        double minimum9_full;
                                        minimum9_full = Convert.ToDouble(readere.GetValue(1))+2400000.0;
                                        //DateTime MODIFIED;
                                        //if (MODIFIED_BRIGHT > MODIFIED_ELEMENT)
                                        //    MODIFIED = MODIFIED_BRIGHT;
                                        //else
                                        //    MODIFIED = MODIFIED_ELEMENT;
                                        int year_NOW = DateTime.Now.Year;
                                        int year_MODIFIED = ConvertFromJulian(minimum9_full).Year;
                                        rating = (year_NOW - year_MODIFIED).ToString();
                                        rating = rating.PadLeft(4);
                                        resultline = resultline + rating;
                                        resultline = resultline + " ";

                                        //24 31  RA2000 Equatorial co - ordinates in equinox 2000.0 - right ascension(hhmmss.s)
                                        string ra2000;
                                        string rs1s="00.0";
                                        if (rs1.ToString() != string.Empty)
                                        {
                                            string before = rs1.ToString().Split(',')[0];
                                            string after = "";
                                            if (rs1.ToString().Contains(','))
                                                after = rs1.ToString().Split(',')[1];
                                            before = before.PadLeft(2, '0');
                                            after = after.PadRight(1, '0');
                                            after = after.Substring(0, Math.Min(1, after.ToString().Length));
                                            rs1s = before + "." + after;
                                        }
                                        ra2000 = rh1.ToString().PadLeft(2, '0') + rm1.ToString().PadLeft(2, '0') + rs1s;
                                        resultline = resultline + ra2000;
                                        resultline = resultline + " ";

                                        //33 39  DE2000 Equatorial co - ordinates in equinox 2000.0 - declination(+ddmmss)
                                        string dec2000;
                                        string ds1s = "00";
                                        if (ds1.ToString() != string.Empty)
                                        {
                                            string before = ds1.ToString().Split(',')[0];
                                            string after = "";
                                            if (ds1.ToString().Contains(','))
                                                after = ds1.ToString().Split(',')[1];
                                            before = before.PadLeft(2, '0');
                                            after = after.PadRight(1, '0');
                                            after = after.Substring(0, Math.Min(1, after.ToString().Length));
                                            ds1s = before;
                                        }
                                        dec2000 = dsi1.ToString().PadRight(1,'+')+dd1.ToString().PadLeft(2, '0') + dm1.ToString().PadLeft(2, '0') + ds1s;
                                        resultline = resultline + dec2000;
                                        resultline = resultline + " ";

                                        //41 51  M0 One of light elements represented by the Julian date of the basic light minimum(epoch E = 0) with the front - end figures 24... omitted.
                                        double minimum9;
                                        minimum9 = Convert.ToDouble(readere.GetValue(1));
                                        string M0;
                                        M0 = minimum9.ToString().Substring(0, Math.Min(10, minimum9.ToString().Length)).PadRight(11, '0');
                                        M0=M0.Replace(',', '.');
                                        resultline = resultline + M0;
                                        resultline = resultline + " ";

                                        //53 64  PER Period P of light changes in days
                                        double period;
                                        period = Convert.ToDouble(readere.GetValue(2));
                                        string PER;
                                        PER = period.ToString().Substring(0, Math.Min(11, period.ToString().Length)).PadRight(12, '0');
                                        PER=PER.Replace(',', '.');
                                        resultline = resultline + PER;
                                        resultline = resultline + " ";

                                        //66 70  MAX Brightness of variable star in maximum(in magnitudes)
                                        string MAX="";
                                        if (maxp.ToString() != string.Empty)
                                        {
                                            string before = maxp.ToString().Split(',')[0];
                                            string after = "";
                                            if (maxp.ToString().Contains(','))
                                                after = maxp.ToString().Split(',')[1];
                                            before = before.PadLeft(2, ' ');
                                            after = after.PadRight(2, '0');
                                            after = after.Substring(0, Math.Min(2, after.ToString().Length));
                                            MAX = before + "." + after;
                                        }
                                        //MAX = maxp.ToString().Split(',')[0].PadLeft(2, ' ') + "." + maxp.ToString().Split(',')[1].Substring(0, Math.Min(2, maxp.ToString().Length)).PadRight(2, '0');
                                        resultline = resultline + MAX;
                                        resultline = resultline + " ";

                                        //72 76  PRIM Brightness of variable star in primary light minimum(in magnitudes)
                                        string PRIM="";
                                        //PRIM = minp.ToString().Split(',')[0].PadLeft(2, ' ') + "." + minp.ToString().Split(',')[1].Substring(0, Math.Min(2, minp.ToString().Length)).PadRight(2, '0');
                                        //string MAX = "";
                                        if (minp.ToString() != string.Empty)
                                        {
                                            string before = minp.ToString().Split(',')[0];
                                            string after = "";
                                            if (minp.ToString().Contains(','))
                                                after = minp.ToString().Split(',')[1];
                                            before = before.PadLeft(2, ' ');
                                            after = after.PadRight(2, '0');
                                            after = after.Substring(0, Math.Min(2, after.ToString().Length));
                                            PRIM = before + "." + after;
                                        }
                                        resultline = resultline + PRIM;
                                        resultline = resultline + " ";

                                        //78 82  SEK Brightness of variable star in secondary light minimum(in magnitudes)
                                        string SEK="";
                                        //SEK = mins.ToString().Split(',')[0].PadLeft(2, ' ') + "." + mins.ToString().Split(',')[1].Substring(0, Math.Min(2, mins.ToString().Length)).PadRight(2, '0');
                                        if (mins.ToString() != string.Empty)
                                        {
                                            string before = mins.ToString().Split(',')[0];
                                            string after = "";
                                            if (mins.ToString().Contains(','))
                                                after = mins.ToString().Split(',')[1];
                                            before = before.PadLeft(2, ' ');
                                            after = after.PadRight(2, '0');
                                            after = after.Substring(0, Math.Min(2, after.ToString().Length));
                                            SEK = before + "." + after;
                                        }
                                        resultline = resultline + SEK;
                                        resultline = resultline + " ";

                                        //84 85  BRANCH The photometric system in which Max, Min and Sec are reported. The main codes are P(photographic magnitudes), V(visual, photovisual, or Johnson's V), "Rc" (R magnitudes in Cousins' I system), "Hp"(Hipparcos magnitudes) and C(unfiltered CCD observations)
                                        string BRANCH;
                                        BRANCH = "V".PadRight(2);
                                        resultline = resultline + BRANCH;
                                        resultline = resultline + " ";

                                        //87 107  ZDROJ Source of elements of light changes
                                        string ZDROJ;
                                        ZDROJ = Convert.ToString(readere.GetValue(3));
                                        ZDROJ = ZDROJ.PadRight(21);
                                        resultline = resultline + ZDROJ;
                                        resultline = resultline + " ";

                                        //109 112  VD Duration of eclipse in hours(for Algol like stars) ; the letter "B" or "W" is given for b Lyrae stars or W UMa stars, respectively.The sign "*" means variability of given value(see Notes).The sign ":" indicates possible uncertainty in given value.
                                        CD = (period * CD * 24) / 1000;
                                        string VD="";
                                        if (CD != 0.0)
                                        {
                                            VD = CD.ToString();
                                        }
                                        else
                                        {
                                            if (type.Contains("EW"))
                                                VD = "W";
                                            if (type.Contains("EC"))
                                                VD = "B";
                                            if (!(type.Contains("EW") || !type.Contains("EC")))
                                                VD = CD.ToString();
                                        }
                                        VD = VD.ToString().PadRight(4);
                                        VD = VD.Substring(0, Math.Min(4, VD.ToString().Length));
                                        VD = VD.Replace(',', '.');
                                        resultline = resultline + VD;
                                        resultline = resultline + " ";

                                        //114 117  MD  Duration of constant phase in minimum light in hours.The sign ":" indicates potential uncertainty in given value
                                        SD = (period * SD * 24) / 1000;
                                        string MD;
                                        if (SD != 0.0)
                                            MD = SD.ToString();
                                        else
                                            MD = "0.0".PadRight(4);
                                        MD = MD.ToString().PadRight(4);
                                        MD = MD.Substring(0, Math.Min(4, MD.ToString().Length));
                                        MD = MD.Replace(',', '.');
                                        resultline = resultline + MD;
                                        resultline = resultline + " ";

                                        //119 126  CHART  Availability of the chart.
                                        string CHART;
                                        CHART = "PROSPER".PadRight(8);
                                        resultline = resultline + CHART;
                                        resultline = resultline + " ";

                                        //128 137  ATRAKTIV  Astrophysical attractiveness of system.
                                        string ATRAKTIV;
                                        ATRAKTIV = "      ";
                                        ATRAKTIV = ATRAKTIV.PadRight(10);
                                        resultline = resultline + ATRAKTIV;
                                        resultline = resultline + " ";

                                        //139 153  POZNAMKA  Remarks.
                                        string POZNAMKA;
                                        string note_el = Convert.ToString(readere.GetValue(4));
                                        POZNAMKA = "               ";
                                        if ((kind=="R") || (kind == "r"))
                                        {
                                            POZNAMKA = kind+"-";
                                        }
                                        POZNAMKA = POZNAMKA + note_id + "-" + note_el;
                                        POZNAMKA = POZNAMKA.PadRight(15);
                                        POZNAMKA=POZNAMKA.Substring(0, Math.Min(15, POZNAMKA.ToString().Length));
                                        resultline = resultline + POZNAMKA;
                                        resultline = resultline + " ";

                                        //155 165  GSC  The Guide Star Catalogue identification number is given in form XXXXX.XXXXX.
                                        string GSC;
                                        //GSC = "           ";
                                        GSC = gsc_id.ToString();
                                        if ((GSC.ToString() != string.Empty)||
                                            (GSC.ToString()!= "-1") ||
                                            (!GSC.ToString().Contains("-1")) ||
                                            (GSC.ToString()!="0")||
                                            (GSC.ToString()!= "-2"))
                                        {
                                            string before = GSC.ToString().Split(',')[0];
                                            string after = "";
                                            if (GSC.ToString().Contains(','))
                                                after = GSC.ToString().Split(',')[1];
                                            else
                                                GSC = "           ";
                                            before = before.PadLeft(5, '0');
                                            after = after.PadRight(5, '0');
                                            after = after.Substring(0, Math.Min(5, after.ToString().Length));
                                            GSC = before + "." + after;
                                        }
                                        if ((GSC.ToString() == string.Empty) ||
                                            (GSC.ToString() == "-1") ||
                                            (GSC.ToString().Contains("-1")) ||
                                            (GSC.ToString() == "0") || 
                                            (GSC.ToString().Contains("-2")))
                                        {
                                            GSC = "           ";
                                        }
                                        resultline = resultline + GSC;
                                        resultline = resultline + " ";

                                        //167 171  ZMENA  The sign "*" means change in light elements relative to previous catalogue BRKA 2004 Z and "N" means the new added star.
                                        string ZMENA;
                                        ZMENA = "     ";
                                        resultline = resultline + ZMENA;
                                        //resultline = resultline + " ";



                                        //string objname = lastline.Substring(0, 9);
                                        //objname= Regex.Replace(objname, @"\s+", " ");



                                        //this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + resultline + "\n";
                                        this.richTextBoxOutput.Text = resultline + "\n";
                                        this.richTextBoxOutput.Refresh();
                                        fileWriter1.WriteLine(resultline);
                                        resultline = "";
                                    }
                                }
                                else
                                {
                                    //ELEMENT has no rows for object
                                    richTextBoxOutput.Text = "No results found in ELEMENT for " + starname_backup + " " + cons_backup;
                                    fileWriter2.WriteLine("No results found in ELEMENT for " + starname_backup + " " + cons_backup);
                                }

                                readere.Close();
                            }

                            nr++;

                        }
                    }
                    else
                    {
                        //IDENTIF has no rows
                        richTextBoxOutput.Text = "No results found in IDENTIF";
                        fileWriter2.WriteLine("No results found in IDENTIF");
                    }

                    reader.Close();

                    this.richTextBoxOutput.Text = "DONE !" + "\n";

                    fileWriter1.Flush();
                    fileWriter1.Close();
                    fileWriter1.Dispose();
                    fileWriter1 = null;

                    fileWriter2.Flush();
                    fileWriter2.Close();
                    fileWriter2.Dispose();
                    fileWriter2 = null;
                }
            }
            catch (Exception ex)
            {
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\n" + ex.ToString();
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\n" + ex.Data.ToString();
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\n" + ex.StackTrace.ToString();
                this.labelStatus.Text = "Status: ERROR !";
                this.labelStatus.ForeColor = Color.Orange;
                this.labelStatus.Refresh();
                //this.labelStatus.Dispose();
            }
            finally
            {
                this.labelStatus.Refresh();
            }
        }

        private void Predpovedi()
        {
            string cons_backup = "";
            string starname_backup = "";
            try
            {
                string filename = Path.GetFileNameWithoutExtension(this.textBoxInput.Text);
                string directoryname = Path.GetDirectoryName(this.textBoxInput.Text);
                string outputfile1 = directoryname + @"\" + filename + "catalog.dat";
                string outputfile2 = directoryname + @"\" + filename + "errorpredpovedi.txt";

                StreamWriter fileWriter1 = new StreamWriter(outputfile1, true);
                StreamWriter fileWriter2 = new StreamWriter(outputfile2, true);

                //StreamReader file = new StreamReader(this.textBoxInput.Text);

                string header = @"DESIGNATION,RA,DEC,PERIOD,HJD0P,HJD0S,MAX,MIN/AMP,TYPE,OBS,NOTE";
                this.richTextBoxOutput.Text = header + "\n";
                fileWriter1.WriteLine(header);

                string resultline = "";

                OleDbConnection con; con = new OleDbConnection("Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + Application.StartupPath + "\\db\\Paschke.accdb;");
                con.Open();
                //string strqry = "SELECT ID,ObjName,RAh,RAm,RAs,Decd,Decm,Decs FROM Objects;";
                string strqry = "SELECT CONS, STARNAME, RH1, RM1, RS1, DSI1, DD1, DM1, DS1, TYPE, GSC_ID, NOTE_ID FROM IDENTIF WHERE OCGATE>#01/01/1900#;";
                OleDbCommand cmd = new OleDbCommand(strqry, con);
                cmd.CommandText = strqry;

                //Read in all elements in IDENTIF
                using (OleDbDataReader reader = cmd.ExecuteReader())
                {
                    if (reader.HasRows)
                    {
                        this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\nDatabase entries:\n";

                        int nr = 0;
                        while (reader.Read() && nr < 100000)
                        {
                            //int id = reader.GetInt32(0);
                            //string name = reader.GetString(1);



                            //double ra = reader.GetInt32(2) + (reader.GetInt32(3) / 60) + (reader.GetDouble(4) / 3600);
                            //double dec = reader.GetInt32(5) + (reader.GetInt32(6) / 60) + (reader.GetDouble(7) / 3600);

                            //Tuple<int, string, int, int> obj = new Tuple<int, string, int, int>(id, name, ra, dec);
                            //objlist.Add(new Tuple<int, string, double, double>(id, name, ra, dec));
                            //objlist.Add(Tuple.Create(id, name, ra, dec));

                            string cons;
                            if ((reader.GetValue(0) != DBNull.Value) && !(reader.GetValue(0) is DbException))
                                cons = Convert.ToString(reader.GetValue(0));
                            else
                                cons = "XX";
                            string starname;
                            if ((reader.GetValue(1) != DBNull.Value) && !(reader.GetValue(1) is DbException))
                                starname = Convert.ToString(reader.GetValue(1));
                            else
                                starname = "XX";
                            double rh1;
                            if ((reader.GetValue(2) != DBNull.Value) && !(reader.GetValue(2) is DbException))
                                rh1 = Convert.ToDouble(reader.GetValue(2));
                            else
                                rh1 = Convert.ToDouble("");
                            double rm1;
                            if ((reader.GetValue(3) != DBNull.Value) && !(reader.GetValue(3) is DbException))
                                rm1 = Convert.ToDouble(reader.GetValue(3));
                            else
                                rm1 = Convert.ToDouble("");
                            double rs1;
                            if ((reader.GetValue(4) != DBNull.Value) && !(reader.GetValue(4) is DbException))
                                rs1 = Convert.ToDouble(reader.GetValue(4));
                            else
                                rs1 = Convert.ToDouble("");
                            string dsi1;
                            if ((reader.GetValue(5) != DBNull.Value) && !(reader.GetValue(5) is DbException))
                                dsi1 = Convert.ToString(reader.GetValue(5));
                            else
                                dsi1 = "+";
                            double dd1;
                            if ((reader.GetValue(6) != DBNull.Value) && !(reader.GetValue(6) is DbException))
                                dd1 = Convert.ToDouble(reader.GetValue(6));
                            else
                                dd1 = Convert.ToDouble("");
                            double dm1;
                            if ((reader.GetValue(7) != DBNull.Value) && !(reader.GetValue(7) is DbException))
                                dm1 = Convert.ToDouble(reader.GetValue(7));
                            else
                                dm1 = Convert.ToDouble("");
                            double ds1;
                            if ((reader.GetValue(8) != DBNull.Value) && !(reader.GetValue(8) is DbException))
                                ds1 = Convert.ToDouble(reader.GetValue(8));
                            else
                                ds1 = Convert.ToDouble("");
                            string type;
                            if ((reader.GetValue(9) != DBNull.Value) && !(reader.GetValue(9) is DbException))
                                type = Convert.ToString(reader.GetValue(9));
                            else
                                type = "XX";
                            double gsc_id;
                            if ((reader.GetValue(10) != DBNull.Value) && !(reader.GetValue(10) is DbException))
                                gsc_id = Convert.ToDouble(reader.GetValue(10));
                            else
                                gsc_id = Convert.ToDouble("");
                            string note_id;
                            if ((reader.GetValue(11) != DBNull.Value) && !(reader.GetValue(11) is DbException))
                                note_id = Convert.ToString(reader.GetValue(11));
                            else
                                note_id = "";

                            starname_backup = starname;
                            cons_backup = cons;

                            this.richTextBoxOutput.Text = richTextBoxOutput.Text + "starname:" + starname + "cons:" + cons + "rh1:" + rh1 + "rm1:" + rm1 + "rs1" + rs1;
                            //fileWriter1.WriteLine("starname:" + starname + "cons:" + cons + "rh1:" + rh1 + "rm1:" + rm1 + "rs1" + rs1);

                            double maxp = 99.9;
                            double minp = 99.9;
                            double mins = 99.9;
                            double CD = 0.0;                  //hours
                            double SD = 0.0;                  //hours
                            DateTime MODIFIED_BRIGHT = DateTime.MinValue;

                            string strqryb = "SELECT MAXP, MINP, MINS, CD, SD, MODIFIED FROM BRIGHT WHERE CONS='" + cons + "' AND STARNAME='" + starname + "';";
                            //fileWriter1.WriteLine("strqryb:"+strqryb);
                            OleDbCommand cmdb = new OleDbCommand(strqryb, con);
                            cmdb.CommandText = strqryb;

                            //Read in elements of BRIGHT
                            using (OleDbDataReader readerb = cmdb.ExecuteReader())
                            {
                                if (readerb.HasRows)
                                {
                                    //this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\nDatabase entries:\n";
                                    while (readerb.Read())
                                    {
                                        //Assuming there is only 1 result, otherwise take last entry for BRIGHT table
                                        if ((readerb.GetValue(0) != DBNull.Value) && !(readerb.GetValue(0) is DbException))
                                            maxp = Convert.ToDouble(readerb.GetValue(0));
                                        else
                                            maxp = 99.9;
                                        if ((readerb.GetValue(1) != DBNull.Value) && !(readerb.GetValue(1) is DbException))
                                            minp = Convert.ToDouble(readerb.GetValue(1));
                                        else
                                            minp = 99.9;
                                        if ((readerb.GetValue(2) != DBNull.Value) && !(readerb.GetValue(2) is DbException))
                                            mins = Convert.ToDouble(readerb.GetValue(2));
                                        else
                                            mins = 99.9;
                                        if ((readerb.GetValue(3) != DBNull.Value) && !(readerb.GetValue(3) is DbException))
                                            CD = Convert.ToDouble(readerb.GetValue(3));
                                        else
                                            CD = 0.0;
                                        if ((readerb.GetValue(4) != DBNull.Value) && !(readerb.GetValue(4) is DbException))
                                            SD = Convert.ToDouble(readerb.GetValue(4));
                                        else
                                            SD = 0.0;
                                        if ((readerb.GetValue(5) != DBNull.Value) && !(readerb.GetValue(5) is DbException))
                                            MODIFIED_BRIGHT = Convert.ToDateTime(readerb.GetValue(5));
                                        //else
                                        //    MODIFIED_BRIGHT = 0.0;
                                    }
                                }
                                else
                                {
                                    //BRIGHT has no rows
                                    richTextBoxOutput.Text = "No results found in BRIGHT for " + starname_backup + " " + cons_backup;
                                    fileWriter2.WriteLine("No results found in BRIGHT for " + starname_backup + " " + cons_backup);
                                }

                                readerb.Close();
                            }

                            string strqrye = "SELECT KIND, MINIMUM9, PERIOD, ELSOUR, NOTE_EL, MODIFIED FROM ELEMENT WHERE CONS='" + cons + "' AND STARNAME='" + starname + "' AND (KIND='p' OR KIND='R' OR KIND='r');";
                            //fileWriter1.WriteLine("strqrye:" + strqrye);
                            OleDbCommand cmde = new OleDbCommand(strqrye, con);
                            cmde.CommandText = strqrye;

                            //Read in elements of ELEMENT
                            using (OleDbDataReader readere = cmde.ExecuteReader())
                            {
                                if (readere.HasRows)
                                {
                                    //this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\nDatabase entries:\n";
                                    int R = 0;
                                    while (readere.Read())
                                    {
                                        //ISSUE: multiple elements for 1 object: p, s, R
                                        //narrowing down ...

                                        //1 till 1
                                        //resultline = resultline + " ";

                                        //2 13  NAME  Star's designation. Colon in the name denotes that data concern to secondary minimum of the variable.  
                                        string name;
                                        /*
                                        if (firstline.Substring(47, 3) == "SEC")
                                        {
                                            name = firstlineList[0].ToString() + ":" + firstlineList[1].ToString();
                                        }
                                        else
                                        {
                                            name = firstlineList[0].ToString() + " " + firstlineList[1].ToString();
                                        }
                                        */

                                        string kind;
                                        kind = Convert.ToString(readere.GetValue(0));

                                        /*
                                        if (kind == "p")
                                        {
                                            name = starname + " " + cons;
                                        }
                                        if (kind == "s")
                                        {
                                            name = starname + ":" + cons;
                                        }
                                        if ((kind == "R") || (kind == "r"))
                                        {
                                            name = starname + " " + cons;
                                        }
                                        else
                                        {
                                            name = starname + "X" + cons;
                                        }
                                        */

                                        /*
                                        switch (kind)
                                        {
                                            case "p":
                                                name = starname + " " + cons;
                                                break;
                                            case "s":
                                                int breakoff = 8;
                                                if (starname.Length > breakoff)
                                                {
                                                    name = "";
                                                    bool prev_is_letter = true;
                                                    bool period_taken = false;
                                                    for (int i = 0; i < starname.Length; i++)
                                                    {
                                                        char c = starname[i];
                                                        if (Char.IsLetter(c))
                                                        {
                                                            name = name + c.ToString();
                                                            prev_is_letter = true;
                                                        }
                                                        else
                                                        {
                                                            if (prev_is_letter && !period_taken)
                                                            {
                                                                //change letter to nr: add :
                                                                name = name + ":" + c.ToString();
                                                                period_taken = true;
                                                            }
                                                            if (i == (breakoff - 1) && !period_taken)
                                                            {
                                                                //last character: add : anyway
                                                                name = name + ":" + c.ToString();
                                                                period_taken = true;
                                                            }
                                                            if (prev_is_letter && period_taken)
                                                            {
                                                                //: is already taken, write nr
                                                                name = name + c.ToString();
                                                            }
                                                            if (!prev_is_letter && period_taken)
                                                            {
                                                                //last character not number and : already taken
                                                                name = name + c.ToString();
                                                            }
                                                            prev_is_letter = false;
                                                        }

                                                    }
                                                }
                                                else
                                                {
                                                    name = starname + ":" + cons;
                                                }
                                                break;
                                            case "R":
                                                name = starname + " " + cons;
                                                R++;
                                                break;
                                            case "r":
                                                name = starname + " " + cons;
                                                R++;
                                                break;
                                            default:
                                                name = starname + " " + cons;
                                                break;

                                        }
                                        */

                                        //name = name.PadRight(12);
                                        name = starname + " " + cons;
                                        //name = name.Substring(0, Math.Min(12, name.ToString().Length));
                                        resultline = name + ",";

                                        //15 17  TYPE The categorization of the star in B.R.N.O.observational program. For custom catalogues, you can optionally enter any text. Note: type of variability is stored in the VD.  
                                        //string type = "10";

                                        /*
                                        switch (type.Substring(0, Math.Min(2, type.ToString().Length)))
                                        {
                                            case "EA":
                                                type = "EA";
                                                break;
                                            case "EB":
                                                type = "EB";
                                                break;
                                            case "EC":
                                                type = "EC";
                                                break;
                                            case "ED":
                                                type = "ED";
                                                break;
                                            case "EW":
                                                type = "EW";
                                                break;
                                            case "RR":
                                                type = "RR";
                                                break;
                                            default:
                                                switch (type)
                                                {
                                                    case "HADS":
                                                        type = "HDS";
                                                        break;
                                                    case "PUL":
                                                        type = "PUL";
                                                        break;
                                                    case "CEP":
                                                        type = "CEP";
                                                        break;
                                                    case "DSCT":
                                                        type = "DST";
                                                        break;
                                                    default:
                                                        type = "ntd";       //not defined...
                                                        break;
                                                }
                                                //
                                                break;
                                        }
                                        type = type.Substring(0, Math.Min(3, type.ToString().Length));
                                        type = type.PadRight(3);
                                        resultline = resultline + type;
                                        resultline = resultline + " ";
                                        */


                                        //19 22  RATING Given ratings are measures of usefulness of mainly visual observations.
                                        //string rating = "10";
                                        //DateTime MODIFIED_ELEMENT = DateTime.MinValue;
                                        //if ((readere.GetValue(5) != DBNull.Value) && !(readere.GetValue(5) is DbException))
                                        //    MODIFIED_ELEMENT = Convert.ToDateTime(readere.GetValue(5));
                                        //else
                                        //    MODIFIED_ELEMENT = 0.0;
                                        //ALGORITHM: select the highest change and assign 1 passed year the field
                                        //DateTime MODIFIED;
                                        //if (MODIFIED_BRIGHT > MODIFIED_ELEMENT)
                                        //    MODIFIED = MODIFIED_BRIGHT;
                                        //else
                                        //    MODIFIED = MODIFIED_ELEMENT;
                                        //int year_NOW = DateTime.Now.Year;
                                        //int year_MODIFIED = MODIFIED.Year;
                                        //rating = (year_NOW - year_MODIFIED).ToString();
                                        //rating = rating.PadLeft(4);
                                        //resultline = resultline + rating;
                                        //resultline = resultline + " ";

                                        //24 31  RA2000 Equatorial co - ordinates in equinox 2000.0 - right ascension(hhmmss.s)
                                        string ra2000;
                                        string rs1s = "00.0";
                                        if (rs1.ToString() != string.Empty)
                                        {
                                            /*
                                            string before = rs1.ToString().Split(',')[0];
                                            string after = "";
                                            if (rs1.ToString().Contains(','))
                                                after = rs1.ToString().Split(',')[1];
                                            before = before.PadLeft(2, '0');
                                            after = after.PadRight(1, '0');
                                            after = after.Substring(0, Math.Min(1, after.ToString().Length));
                                            rs1s = before + "." + after;
                                            */
                                            rs1s = rs1.ToString().Replace(',', '.');
                                        }
                                        ra2000 = rh1.ToString().PadLeft(2, '0') + ":" + rm1.ToString().PadLeft(2, '0') + ":" + rs1s;
                                        resultline = resultline + ra2000;
                                        resultline = resultline + ",";

                                        //33 39  DE2000 Equatorial co - ordinates in equinox 2000.0 - declination(+ddmmss)
                                        string dec2000;
                                        string ds1s = "00";
                                        if (ds1.ToString() != string.Empty)
                                        {
                                            /*
                                            string before = ds1.ToString().Split(',')[0];
                                            string after = "";
                                            if (ds1.ToString().Contains(','))
                                                after = ds1.ToString().Split(',')[1];
                                            before = before.PadLeft(2, '0');
                                            after = after.PadRight(1, '0');
                                            after = after.Substring(0, Math.Min(1, after.ToString().Length));
                                            ds1s = before;
                                            */
                                            ds1s = ds1.ToString().Replace(',', '.');
                                        }
                                        dec2000 = dsi1.ToString().PadRight(1, '+') + dd1.ToString().PadLeft(2, '0') + ":" + dm1.ToString().PadLeft(2, '0') + ":" + ds1s;
                                        resultline = resultline + dec2000;
                                        resultline = resultline + ",";

                                        //53 64  PER Period P of light changes in days
                                        double period;
                                        period = Convert.ToDouble(readere.GetValue(2));
                                        string PER = period.ToString();
                                        //PER = period.ToString().Substring(0, Math.Min(11, period.ToString().Length)).PadRight(12, '0');
                                        PER = PER.Replace(',', '.');
                                        resultline = resultline + PER;
                                        resultline = resultline + ",";

                                        //41 51  M0 One of light elements represented by the Julian date of the basic light minimum(epoch E = 0) with the front - end figures 24... omitted.
                                        double minimum9;
                                        minimum9 = Convert.ToDouble(readere.GetValue(1));
                                        string M0 = minimum9.ToString();
                                        //M0 = minimum9.ToString().Substring(0, Math.Min(10, minimum9.ToString().Length)).PadRight(11, '0');
                                        M0 = M0.Replace(',', '.');
                                        resultline = resultline + M0;
                                        resultline = resultline + ",";

                                        //HJD0S
                                        string periods="";
                                        string strqrys = "SELECT MINIMUM9 FROM ELEMENT WHERE CONS='" + cons + "' AND STARNAME='" + starname + "' AND KIND='s';";
                                        //fileWriter1.WriteLine("strqrye:" + strqrye);
                                        OleDbCommand cmds = new OleDbCommand(strqrys, con);
                                        cmds.CommandText = strqrys;
                                        //Read in elements of ELEMENT
                                        using (OleDbDataReader readers = cmds.ExecuteReader())
                                        {
                                            while (readers.Read())
                                            {
                                                if (readers.HasRows)
                                                {
                                                    double minimumsec;
                                                    if ((readers.GetValue(0) != DBNull.Value) && !(readers.GetValue(0) is DbException))
                                                        minimumsec = Convert.ToDouble(readers.GetValue(0));
                                                    else
                                                        minimumsec = 0.0;
                                                    
                                                    periods = minimumsec.ToString().Replace(',', '.');
                                                }
                                                else
                                                {
                                                    periods = "";
                                                }
                                            }
                                            readers.Close();
                                        }
                                        if (periods == "")
                                        {
                                            resultline = resultline+"0,";
                                        }
                                        else
                                        {
                                            resultline = resultline+periods + ",";
                                        }
                                        

                                        //66 70  MAX Brightness of variable star in maximum(in magnitudes)
                                        string MAX = "";
                                        if (maxp.ToString() != string.Empty)
                                        {
                                            /*
                                            string before = maxp.ToString().Split(',')[0];
                                            string after = "";
                                            if (maxp.ToString().Contains(','))
                                                after = maxp.ToString().Split(',')[1];
                                            before = before.PadLeft(2, ' ');
                                            after = after.PadRight(2, '0');
                                            after = after.Substring(0, Math.Min(2, after.ToString().Length));
                                            MAX = before + "." + after;
                                            */
                                            MAX = maxp.ToString().Replace(',', '.');
                                        }
                                        //MAX = maxp.ToString().Split(',')[0].PadLeft(2, ' ') + "." + maxp.ToString().Split(',')[1].Substring(0, Math.Min(2, maxp.ToString().Length)).PadRight(2, '0');
                                        resultline = resultline + MAX;
                                        resultline = resultline + ",";

                                        //72 76  PRIM Brightness of variable star in primary light minimum(in magnitudes)
                                        string PRIM = "";
                                        //PRIM = minp.ToString().Split(',')[0].PadLeft(2, ' ') + "." + minp.ToString().Split(',')[1].Substring(0, Math.Min(2, minp.ToString().Length)).PadRight(2, '0');
                                        //string MAX = "";
                                        if (minp.ToString() != string.Empty)
                                        {
                                            /*
                                            string before = minp.ToString().Split(',')[0];
                                            string after = "";
                                            if (minp.ToString().Contains(','))
                                                after = minp.ToString().Split(',')[1];
                                            before = before.PadLeft(2, ' ');
                                            after = after.PadRight(2, '0');
                                            after = after.Substring(0, Math.Min(2, after.ToString().Length));
                                            PRIM = before + "." + after;
                                            */
                                            PRIM = minp.ToString().Replace(',', '.');
                                        }
                                        resultline = resultline + PRIM;
                                        resultline = resultline + ",";

                                        //78 82  SEK Brightness of variable star in secondary light minimum(in magnitudes)
                                        //string SEK = "";
                                        //SEK = mins.ToString().Split(',')[0].PadLeft(2, ' ') + "." + mins.ToString().Split(',')[1].Substring(0, Math.Min(2, mins.ToString().Length)).PadRight(2, '0');
                                        /*
                                        if (mins.ToString() != string.Empty)
                                        {
                                            string before = mins.ToString().Split(',')[0];
                                            string after = "";
                                            if (mins.ToString().Contains(','))
                                                after = mins.ToString().Split(',')[1];
                                            before = before.PadLeft(2, ' ');
                                            after = after.PadRight(2, '0');
                                            after = after.Substring(0, Math.Min(2, after.ToString().Length));
                                            SEK = before + "." + after;
                                        }
                                        resultline = resultline + SEK;
                                        resultline = resultline + " ";
                                        */

                                        //84 85  BRANCH The photometric system in which Max, Min and Sec are reported. The main codes are P(photographic magnitudes), V(visual, photovisual, or Johnson's V), "Rc" (R magnitudes in Cousins' I system), "Hp"(Hipparcos magnitudes) and C(unfiltered CCD observations)
                                        //string BRANCH;
                                        //BRANCH = "V".PadRight(2);
                                        //resultline = resultline + BRANCH;
                                        //resultline = resultline + " ";

                                        //Type
                                        if (type != "")
                                        {
                                            type = type.Replace(',', '_');
                                            type = type.Replace(' ', '_');
                                            resultline =resultline + type + ",";
                                        }
                                        else
                                        {
                                            resultline = resultline + "0,";
                                        }

                                        //obs
                                        resultline = resultline + "2,";

                                        //87 107  ZDROJ Source of elements of light changes
                                        string ZDROJ;
                                        ZDROJ = Convert.ToString(readere.GetValue(3));
                                        //ZDROJ = ZDROJ.PadRight(21);
                                        if (ZDROJ != "")
                                        {
                                            ZDROJ = ZDROJ.Replace(',', '_');
                                            ZDROJ = ZDROJ.Replace(' ', '_');
                                            resultline = resultline + ZDROJ;
                                        }
                                        else
                                        {
                                            resultline = resultline + "none";
                                        }
                                        
                                        //resultline = resultline;

                                        //109 112  VD Duration of eclipse in hours(for Algol like stars) ; the letter "B" or "W" is given for b Lyrae stars or W UMa stars, respectively.The sign "*" means variability of given value(see Notes).The sign ":" indicates possible uncertainty in given value.
                                        /*
                                        CD = (period * CD * 24) / 1000;
                                        string VD = "";
                                        if (CD != 0.0)
                                        {
                                            VD = CD.ToString();
                                        }
                                        else
                                        {
                                            if (type.Contains("EW"))
                                                VD = "W";
                                            if (type.Contains("EC"))
                                                VD = "B";
                                            if (!(type.Contains("EW") || !type.Contains("EC")))
                                                VD = CD.ToString();
                                        }
                                        VD = VD.ToString().PadRight(4);
                                        VD = VD.Substring(0, Math.Min(4, VD.ToString().Length));
                                        VD = VD.Replace(',', '.');
                                        resultline = resultline + VD;
                                        resultline = resultline + " ";
                                        */

                                        //114 117  MD  Duration of constant phase in minimum light in hours.The sign ":" indicates potential uncertainty in given value
                                        /*
                                        SD = (period * SD * 24) / 1000;
                                        string MD;
                                        if (SD != 0.0)
                                            MD = SD.ToString();
                                        else
                                            MD = "0.0".PadRight(4);
                                        MD = MD.ToString().PadRight(4);
                                        MD = MD.Substring(0, Math.Min(4, MD.ToString().Length));
                                        MD = MD.Replace(',', '.');
                                        resultline = resultline + MD;
                                        resultline = resultline + " ";
                                        */

                                        //119 126  CHART  Availability of the chart.
                                        /*
                                        string CHART;
                                        CHART = "PROSPER".PadRight(8);
                                        resultline = resultline + CHART;
                                        resultline = resultline + " ";
                                        */

                                        //128 137  ATRAKTIV  Astrophysical attractiveness of system.
                                        /*
                                        string ATRAKTIV;
                                        ATRAKTIV = "      ";
                                        ATRAKTIV = ATRAKTIV.PadRight(10);
                                        resultline = resultline + ATRAKTIV;
                                        resultline = resultline + " ";
                                        */

                                        //139 153  POZNAMKA  Remarks.
                                        string POZNAMKA="";
                                        
                                        string note_el = Convert.ToString(readere.GetValue(4));
                                        /*
                                        POZNAMKA = "               ";
                                        if ((kind == "R") || (kind == "r"))
                                        {
                                            POZNAMKA = kind + "-";
                                        }
                                        */
                                        note_id = note_id.Replace(',', '_');
                                        note_id = note_id.Replace(' ', '_');
                                        note_el = note_el.Replace(',', '_');
                                        note_el = note_el.Replace(' ', '_');
                                        POZNAMKA = POZNAMKA +"_"+ note_id +"_"+ note_el;
                                        //POZNAMKA = POZNAMKA.PadRight(15);
                                        //POZNAMKA = POZNAMKA.Substring(0, Math.Min(15, POZNAMKA.ToString().Length));
                                        resultline = resultline + POZNAMKA;
                                        //resultline = resultline + " ";

                                        //155 165  GSC  The Guide Star Catalogue identification number is given in form XXXXX.XXXXX.
                                        //string GSC;
                                        //GSC = "           ";
                                        /*
                                        GSC = gsc_id.ToString();
                                        if ((GSC.ToString() != string.Empty) ||
                                            (GSC.ToString() != "-1") ||
                                            (!GSC.ToString().Contains("-1")) ||
                                            (GSC.ToString() != "0") ||
                                            (GSC.ToString() != "-2"))
                                        {
                                            string before = GSC.ToString().Split(',')[0];
                                            string after = "";
                                            if (GSC.ToString().Contains(','))
                                                after = GSC.ToString().Split(',')[1];
                                            else
                                                GSC = "           ";
                                            before = before.PadLeft(5, '0');
                                            after = after.PadRight(5, '0');
                                            after = after.Substring(0, Math.Min(5, after.ToString().Length));
                                            GSC = before + "." + after;
                                        }
                                        if ((GSC.ToString() == string.Empty) ||
                                            (GSC.ToString() == "-1") ||
                                            (GSC.ToString().Contains("-1")) ||
                                            (GSC.ToString() == "0") ||
                                            (GSC.ToString().Contains("-2")))
                                        {
                                            GSC = "           ";
                                        }
                                        resultline = resultline + GSC;
                                        resultline = resultline + " ";
                                        */

                                        //167 171  ZMENA  The sign "*" means change in light elements relative to previous catalogue BRKA 2004 Z and "N" means the new added star.
                                        //string ZMENA;
                                        //ZMENA = "     ";
                                        //resultline = resultline + ZMENA;
                                        //resultline = resultline + " ";



                                        //string objname = lastline.Substring(0, 9);
                                        //objname= Regex.Replace(objname, @"\s+", " ");



                                        //this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + resultline + "\n";
                                        this.richTextBoxOutput.Text = resultline + "\n";
                                        this.richTextBoxOutput.Refresh();
                                        fileWriter1.WriteLine(resultline);
                                        resultline = "";
                                    }
                                }
                                else
                                {
                                    //ELEMENT has no rows for object
                                    richTextBoxOutput.Text = "No results found in ELEMENT for " + starname_backup + " " + cons_backup;
                                    fileWriter2.WriteLine("No results found in ELEMENT for " + starname_backup + " " + cons_backup);
                                }

                                readere.Close();
                            }

                            nr++;

                        }
                    }
                    else
                    {
                        //IDENTIF has no rows
                        richTextBoxOutput.Text = "No results found in IDENTIF";
                        fileWriter2.WriteLine("No results found in IDENTIF");
                    }

                    reader.Close();

                    this.richTextBoxOutput.Text = "DONE !" + "\n";

                    fileWriter1.Flush();
                    fileWriter1.Close();
                    fileWriter1.Dispose();
                    fileWriter1 = null;

                    fileWriter2.Flush();
                    fileWriter2.Close();
                    fileWriter2.Dispose();
                    fileWriter2 = null;
                }
            }
            catch (Exception ex)
            {
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\n" + ex.ToString();
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\n" + ex.Data.ToString();
                this.richTextBoxOutput.Text = this.richTextBoxOutput.Text + "\n" + ex.StackTrace.ToString();
                this.labelStatus.Text = "Status: ERROR !";
                this.labelStatus.ForeColor = Color.Orange;
                this.labelStatus.Refresh();
                //this.labelStatus.Dispose();
            }
            finally
            {
                this.labelStatus.Refresh();
            }
        }

        public static DateTime ConvertFromJulian(double julianDate)
        {
            
            double unixTime = (julianDate - 2440587.5) * 86400;

            DateTime dtDateTime = new DateTime(1970, 1, 1, 0, 0, 0, 0, System.DateTimeKind.Utc);
            dtDateTime = dtDateTime.AddSeconds(unixTime).ToLocalTime();

            return dtDateTime;
            
            //return DateTime.FromOADate(julianDate);
        }
    }
}
